Ext.define('DSS.component.DeviceSelector', {
    extend : 'DSS.component.Base',
    mixins : {
        observable : 'Ext.util.Observable'
    },
    height : '100%',
    grids : {},
    macPanel : null,
    iDefaultWidth : 150,
    iDefaultHeight : 293,
    filterFunc:null,
    specialDeviceRenderer:null,
    selectedDSID:null,
    setHeight : function(oHeight) {
        this.macPanel.setHeight(oHeight);
        this.iDefaultHeight = oHeight - 2;
        for ( var oKey in this.grids)
            this.grids[oKey].setHeight(this.iDefaultHeight);
        this.callParent(arguments);
    },
    initStores : function() {
        if (Ext.data.StoreManager.lookup('deviceSelector_zoneStore') == null) {
            // Achtung: nur ein control !
            Ext.create('Ext.data.Store', {
                storeId : 'deviceSelector_zoneStore',
                fields : [ 'ZoneID', 'name' , 'passive'],
                proxy : {
                    type : 'memory',
                    reader : {
                        type : 'json',
                        root : 'items'
                    }
                }
            });
            var oArray=dss.staticDataModel.getZoneArray();
            
            for (var iIndex=0;iIndex<oArray.length;iIndex++) {
                Ext.create('Ext.data.Store', {
                    storeId : 'deviceSelector_deviceStore_' + oArray[iIndex].ZoneID,
                    fields : [ 'dSID', 'name', 'icon', 'present' ],
                    proxy : {
                        type : 'memory',
                        reader : {
                            type : 'json',
                            root : 'items'
                        }
                    }
                });
                var oObject={
                        ZoneID:oArray[iIndex].ZoneID,
                        name:oArray[iIndex].name,
                        passive:true
                        };
                
                var oDeviceArray=dss.staticDataModel.getDevicesForZone(oArray[iIndex].ZoneID,0);
                for (var jIndex=0;jIndex<oDeviceArray.length;jIndex++) {
                    var oDeviceObject=oDeviceArray[jIndex];
                    if (oDeviceObject.present) {
                        if (this.filterFunc==null) {
                            oObject.passive = false;                            
                            Ext.data.StoreManager.lookup('deviceSelector_deviceStore_' + oArray[iIndex].ZoneID).add(oDeviceObject);
                        } else
                            if (this.filterFunc(oDeviceArray[jIndex])) {
                                oObject.passive = false;
                                Ext.data.StoreManager.lookup('deviceSelector_deviceStore_' + oArray[iIndex].ZoneID).add(oDeviceObject);
                            }
                    }
                }
                Ext.data.StoreManager.lookup('deviceSelector_zoneStore').add(oObject);
            }
        }
    },   
    defMacGridRenderer : function(val, metadata, record, colIdx, store, view) {
        var fHasIcon=false;
        if (record.get('icon') != null)
        {
            fHasIcon=true;
        }
        var fHasNext=true;
        if (record.get('dSID') != null)    // ist ein device -> kein weiterbutton
        {
            fHasNext=false;
        } 

        var fIsPassive=false;        
        if (record.get('passive')!=null)
            if (record.get('passive'))
                fIsPassive=true;
        
        if (fIsPassive) {
            fHasNext=false;               // spalte disabled -> niemals weiterbutton
        }
        
        var sReturn='<table style="height:16px;"><tr>';
        var iColspaceCounter=3;
        if (fHasIcon)
        {
            iColspaceCounter--;  
            sReturn+='<td><img src="' + record.get('icon') + '" style="margin-right: 5px;"></td>';
        }

        if (fHasNext)
            iColspaceCounter--;
        if (fIsPassive) {
            sReturn+='<td style="color:#888888;" colspan="'+iColspaceCounter+'" width="100%">' + record.get('name') + '</td>';
        } else {
            sReturn+='<td colspan="'+iColspaceCounter+'" width="100%">' + record.get('name') + '</td>';
        }
        if (fHasNext) {
            sReturn+='<td>&gt;</td>';
        }
        return sReturn +'</tr></table>';      
    },
    getDeviceStore : function (zoneID) {
        return Ext.data.StoreManager.lookup('deviceSelector_deviceStore_' + zoneID);
    },
    constructGridLvlDevice : function (zoneID) {
        // construct device grids
        var oDeviceControlStore = Ext.create('Ext.data.Store', {
            fields : [ 'dSID', 'name', 'icon', 'present' ],
            proxy : {
                type : 'memory',
                reader : {
                    type : 'json',
                    root : 'items'
                }
            }
        });

        var oDeviceStore = this.getDeviceStore(zoneID);
        oDeviceStore.each(function(record) {
            var oNewRecord = record.copy()
            if (oNewRecord.get('present'))
                oDeviceControlStore.add(oNewRecord);
        });
        oDeviceControlStore.sort('name');
        this.grids.lvlDevice = Ext.create('Ext.grid.Panel', {
            hideHeaders:true,
            height : this.iDefaultHeight,
            width : 624,
            autoScroll : true,
            viewConfig : {
                stripeRows : true
            },
            store : oDeviceControlStore,
            columns : [ {
                sortable : false,
                hideable : false,
                text : _('device'),
                dataIndex : 'name',
                flex : 1,
                renderer : (this.specialDeviceRenderer!=null)?this.specialDeviceRenderer:this.defMacGridRenderer
            } ]
        });
        this.grids.lvlDevice.parent = this;
        this.grids.lvlDevice.on('itemclick', function(grid, record) {
            this.parent.selectedDSID=record.get('dSID');
        });
        this.macPanel.add(this.grids.lvlDevice);

    
    },
    constructMacGrids : function() {
        // construct room grid
        
        var oZoneControlStore = Ext.create('Ext.data.Store', {
            fields : [ 'ZoneID', 'name' , 'passive'],
            proxy : {
                type : 'memory',
                reader : {
                    type : 'json',
                    root : 'items'
                }
            }
        });

        var oZoneStore = Ext.data.StoreManager.lookup('deviceSelector_zoneStore');
        oZoneStore.each(function(record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('ZoneID') != 0)
                oZoneControlStore.add(oNewRecord);
        });
        oZoneControlStore.sort('name');
        /*oZoneControlStore.add({
            'ZoneID' : -1,
            'name' : _('any room')
        });*/

        this.grids.lvlRoom = Ext.create('Ext.grid.Panel', {
            hideHeaders:true,
            height : this.iDefaultHeight,
            width : this.iDefaultWidth,
            autoScroll : true,
            viewConfig : {
                stripeRows : true
            },
            store : oZoneControlStore,
            columns : [ {
                sortable : false,
                hideable : false,
                text : _('room'),
                dataIndex : 'name',
                flex : 1,
                renderer :function(val, metadata, record, colIdx, store, view) {
                    var fIsPassive=false;        
                    if (record.get('passive')!=null)
                        if (record.get('passive'))
                            fIsPassive=true;
                    
                    var sReturn='<table style="height:16px;"><tr>';
                    if (fIsPassive) {
                        sReturn+='<td style="color:#888888;" colspan="2" width="100%">' + record.get('name') + '</td>';
                    } else {
                        sReturn+='<td width="100%">' + record.get('name') + '</td><td>&gt;</td>';
                    }
                    return sReturn +'</tr></table>';      
                }
            } ]
        });

        this.grids.lvlRoom.parent = this;
        this.grids.lvlRoom.on('itemclick', function(grid, record) {
            this.up().remove(this.up().up().grids.lvlDevice);
            var zoneID = record.get('ZoneID');
            if (zoneID == -1)
                zoneID = 0;
            this.up().up().constructGridLvlDevice(zoneID);
        });
        this.macPanel.add(this.grids.lvlRoom);

    },
    constructor : function (config) {
        if (config.filterFunc!=null)
            this.filterFunc=config.filterFunc;
        else
            this.filterFunc=null;
        if (config.specialDeviceRenderer!=null)
            this.specialDeviceRenderer=config.specialDeviceRenderer;        
        else
            this.specialDeviceRenderer=null;
        this.initStores();

        this.macPanel = Ext.create('Ext.panel.Panel', {
            border : '0',
            autoScroll : true,
            height : 320,
            layout : {
                type : 'hbox'
            },
            scroll : 'horizontal'
        });
        this.constructMacGrids();
        Ext.apply(this, {
            items : [ this.macPanel ]
        });

        this.callParent(arguments);
  
    },
    reset : function() {
        this.setValue(null);
    },
    setValue : function(values) {
        this.macPanel.remove(this.grids.lvlDevice);
        this.selectedDSID=values;
        if (values==null)
        	 this.grids.lvlRoom.getSelectionModel().deselectAll();
        var oDevice=dss.staticDataModel.getDevice(values);
        var oRecordZone = this.grids.lvlRoom.store.find('ZoneID', oDevice.ZoneID,0,false,false,true);
        if (oRecordZone != -1) {
            this.grids.lvlRoom.getSelectionModel().select(oRecordZone);
            this.constructGridLvlDevice(oDevice.ZoneID);
            
            var oRecordDevice = this.grids.lvlDevice.store.find('dSID', values,0,false,false,true);
            if (oRecordDevice != -1) 
                this.grids.lvlDevice.getSelectionModel().select(oRecordDevice);
        }
    },
    isValid : function() {
        return (this.selectedDSID!=null);
    },
    getValue : function() {
        return this.selectedDSID;
    }
});